#include "defs.h"

#include "md5.h"
#include "aes.h"
#include "sha.h"

#include <stdio.h>
#include <string.h>
#include <time.h>
#include <stdlib.h>

void crypto_init(chat *c)
{
  int i;
  sha256_context sc;
  MD5_CTX mc;
  char pp[32];
  char pp2[32];
  time_t tt;
  struct tm *t;
  tt = time(0);
  t = gmtime(&tt);
  memset(c->iv, 0x5a, 16);
  c->iv[0] = (t->tm_year >> 8) & 255;
  c->iv[1] = t->tm_year & 255;
  c->iv[2] = t->tm_mon;
  c->iv[3] = t->tm_mday;
  c->iv_high = (t->tm_hour << 16) | (t->tm_min << 8) | t->tm_sec;
  c->iv_low = 0;

  /* uselessly complex surely... */
  sha256_starts(&sc);
  sha256_update(&sc, (unsigned char *)c->passphrase, strlen(c->passphrase));
  sha256_finish(&sc, (unsigned char *)pp);

  MD5Init(&mc);
  MD5Update(&mc, (unsigned char *)c->passphrase, strlen(c->passphrase));
  MD5Final(&mc);
  sha256_starts(&sc);
  sha256_update(&sc, mc.digest, 16);
  sha256_finish(&sc, (unsigned char *)pp2);

  memset(c->passphrase, 0, strlen(c->passphrase));

  /* user 1:
   * key1: pp..pp  pp2..pp2
   * key2: pp2..pp2  pp..pp
   */
  for (i = 0; i < 16; i++) {
    if (c->id == 1) {
      c->key1.v[i]    ^= pp[i];
      c->key1.v[i+16] ^= pp2[i+16];
      c->key2.v[i]    ^= pp2[i];
      c->key2.v[i+16] ^= pp[i+16];
    } else {
      c->key2.v[i]    ^= pp[i];
      c->key2.v[i+16] ^= pp2[i+16];
      c->key1.v[i]    ^= pp2[i];
      c->key1.v[i+16] ^= pp[i+16];
    }
  }
}

char *cipher(chat *c, char *m)
{
  char iv[16];
  char b[69];
  static char b2[69];
  char h1[16];
  MD5_CTX mc;
  aes_context ac;

  c->iv[16-5] = b[0] = (c->iv_high >> 16) & 255;
  c->iv[16-4] = b[1] = (c->iv_high >> 8) & 255;
  c->iv[16-3] = b[2] = c->iv_high & 255;
  c->iv[16-2] = b[3] = (c->iv_low >> 8) & 255;
  c->iv[16-1] = b[4] = c->iv_low & 255;
  c->iv_low++; c->iv_low &= 65535; if (!c->iv_low) c->iv_high++;
  memcpy(iv, c->iv, 16);
  memcpy(b+5, m, 48);

  /* H(key1 || H(key2 || message)) */
  MD5Init(&mc);
  MD5Update(&mc, (unsigned char *)c->key2.v, 32);
  MD5Update(&mc, (unsigned char *)b, 48+5);
  MD5Final(&mc);
  memcpy(h1, mc.digest, 16);

  MD5Init(&mc);
  MD5Update(&mc, (unsigned char *)c->key1.v, 32);
  MD5Update(&mc, (unsigned char *)h1, 16);
  MD5Final(&mc);

  memcpy(b+5+48, mc.digest, 16);

  LOG("mess before enc:\n");
  {int i;for(i=0;i<69;i++)LOG("%2.2x",(unsigned char)b[i]);LOG("\n");}

  if (aes_setkey_enc(&ac, (unsigned char *)c->key1.v, 256)) R;
  if (aes_crypt_cbc(&ac, AES_ENCRYPT, 64, (unsigned char *)iv,
                    (unsigned char *)b+5, (unsigned char *)b2+5)) R;
  memcpy(b2, b, 5);

  LOG("mess after enc:\n");
  {int i;for(i=0;i<69;i++)LOG("%2.2x",(unsigned char)b2[i]);LOG("\n");}

  return b2;
}

static char output[256+64];

int _decipher(chat *c, char *b, key *k1, key *k2, char *n)
{
  char iv[16];
  char b2[69];
  char h1[16];
  MD5_CTX mc;
  aes_context ac;
  int op = 0;
  struct tm *t;
  time_t tt;

  LOG("mess before dec:\n");
  {int i;for(i=0;i<69;i++)LOG("%2.2x",(unsigned char)b[i]);LOG("\n");}

  c->iv[16-5] = b2[0] = b[0];
  c->iv[16-4] = b2[1] = b[1];
  c->iv[16-3] = b2[2] = b[2];
  c->iv[16-2] = b2[3] = b[3];
  c->iv[16-1] = b2[4] = b[4];
  memcpy(iv, c->iv, 16);
  if (aes_setkey_dec(&ac, (unsigned char *)k1->v, 256)) R;
  if (aes_crypt_cbc(&ac, AES_DECRYPT, 64, (unsigned char *)iv,
                    (unsigned char *)b+5, (unsigned char *)b2+5)) R;

  LOG("mess after dec:\n");
  {int i;for(i=0;i<69;i++)LOG("%2.2x",(unsigned char)b2[i]);LOG("\n");}

  /* H(key1 || H(key2 || message)) */
  MD5Init(&mc);
  MD5Update(&mc, (unsigned char *)k2->v, 32);
  MD5Update(&mc, (unsigned char *)b2, 48+5);
  MD5Final(&mc);
  memcpy(h1, mc.digest, 16);

  MD5Init(&mc);
  MD5Update(&mc, (unsigned char *)k1->v, 32);
  MD5Update(&mc, (unsigned char *)h1, 16);
  MD5Final(&mc);

  if (memcmp(mc.digest, b2+48+5, 16)) return 0;

  tt = time(0);
  t = localtime(&tt);

  sprintf(output, "%2.2d:%2.2d:%2.2d: ", t->tm_hour, t->tm_min, t->tm_sec);
  sprintf(output+10, "%s: ", n);

  op = 11; while (output[op]) op++;

  memcpy(output+op, b2+5, 48);

  return 1;
}

char *decipher(chat *c, char *b)
{
  /* we suppose that chatters' names are less than 256 in length */

  if (_decipher(c, b, &c->key1, &c->key2, c->my_name))
    return output;
  if (_decipher(c, b, &c->key2, &c->key1, c->her_name))
    return output;

  return 0;
}
