#include "defs.h"
#include "gui.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>

/* WHAT? A GLOBAL? INSANITY! */
int _log;

void oom(void)
{
  printf("out of memory\n");
  exit(1);
}

void usage(void)
{
  printf("usage: chat [options]\n");
  printf("options:\n");
  printf(
   "  -f <config file>                        use given configuration file\n");
  printf(
   "  <parameter name> <parameter value>      to overwrite a parameter\n");
  printf("Run 'chat 0' to see the configuration parameters.\n");
  exit(1);
}

int main(int n, char **vv)
{
  chat c;
  gui g;
  passphrase p;
  int who = 0;
  char *conffile = 0;
  char *my_name;
  char *her_name;
  char *connection;
  int i;
  char **v;

  v = malloc(sizeof(char *) * n); if (!v) R;
  for (i = 0; i < n; i++) v[i] = vv[i];

  for (i = 1; i < n; i++) {
    if (!strcmp(v[i], "-h")) usage();
    if (!strcmp(v[i], "-help")) usage();
    if (!strcmp(v[i], "--help")) usage();
    if (!strcmp(v[i], "-f")) {
      if (i+1 == n) usage();
      conffile = v[i+1];
      v[i] = v[i+1] = 0;
      i++;
    }
  }

  read_options(&c.options, conffile, v, n);

  for (i = 1; i < n; i++)
    if (v[i]) {
      int j;
      printf("unknown option %s\n", v[i]);
      printf("current configuration:\n");
      for (j = 0; j < c.options.size; j++) {
        printf("  %s = %s\n", c.options.data[j].name, c.options.data[j].val);
      }
      return 1;
    }

  sscanf(get_option(&c.options, "log"), "%d", &_log);
  if (_log != 0 && _log != 1){printf("log not correct (0 or 1)\n");return 1;}

  my_name = get_option(&c.options, "my_name");
  her_name = get_option(&c.options, "her_name");
  connection = get_option(&c.options, "connection.type");
  if (strcmp(connection, "mathim") &&
      strcmp(connection, "s"))
    {printf("%s: unknown connection type\n",connection);return 1;}

  sscanf(get_option(&c.options, "flash.x"), "%d", &c.flash_x);
  sscanf(get_option(&c.options, "flash.y"), "%d", &c.flash_y);
  sscanf(get_option(&c.options, "flash.width"), "%d", &c.flash_width);
  sscanf(get_option(&c.options, "flash.height"), "%d", &c.flash_height);
  sscanf(get_option(&c.options, "flash.count"), "%d", &c.flash_count);
  sscanf(get_option(&c.options, "flash.on"), "%d", &c.flash_on);
  sscanf(get_option(&c.options, "flash.off"), "%d", &c.flash_off);
  c.flash_color = get_option(&c.options, "flash.color");
  /* some sanity checks for flash stuff */
  if (c.flash_x < 0 || c.flash_x > 10000 ||
      c.flash_y < 0 || c.flash_y > 10000 ||
      c.flash_count < 1 || c.flash_count > 100 ||
      c.flash_on < 1 || c.flash_on > 1000 ||
      c.flash_off < 1 || c.flash_off > 1000)
    {printf("bad flash settings in conf file\n");return 1;}

  if (strlen(my_name)>256){printf("%s: name too long\n",my_name);return 1;}
  if (strlen(her_name)>256){printf("%s: name too long\n",her_name);return 1;}

  c.channel = get_option(&c.options, "connection.mathim.channel");

  sscanf(get_option(&c.options, "personality"), "%d", &who);
  if (who != 1 && who != 2)
    { printf("bad personality in conf file, must be 1 or 2\n"); return 1; }

  c.update_running = 0;
  c.send_running = 0;

  c.send_queue.size = c.send_queue.maxsize = 0;
  c.send_queue.buf = 0;

  c.id = who;

  g_thread_init(0);
  gdk_threads_init();
  gdk_threads_enter();

  c.data_mutex = g_mutex_new();
  c.network_mutex = g_mutex_new();

  gtk_init(0, 0);
  init_passphrase(&p, &c);
  gtk_widget_show_all(p.w);
  gtk_main();

  {
#include "keys.h"
    int i;
    for (i = 0; i < 32; i++) {
      if (who == 1) {
        c.key1.v[i] = key1[i];
        c.key2.v[i] = key2[i];
      } else {
        c.key2.v[i] = key1[i];
        c.key1.v[i] = key2[i];
      }
    }
  }

  init_gui(&g, &c);
  c.gui = &g;

  crypto_init(&c);

  c.m.size = c.m.maxsize = 0;
  c.m.data = 0;

  set_my_name(&c, my_name);
  set_her_name(&c, her_name);

  if (!strcmp(connection, "mathim"))
    mathim_server_init(&c);
  else
    s_server_init(&c);

  c.connect(&c);

  gtk_widget_show_all(g.w);

  gtk_main();
  gdk_threads_leave();

  return 0;
}
