#include "../defs.h"
#include "defs.h"
#include "buffer.h"

#include <curl/curl.h>
#include <string.h>
#include <stdlib.h>

static do_buf recdata;
static do_buf snddata;

static size_t get_data_cb(void *buffer, size_t size, size_t nmemb, void *userp)
{
  add_buffer(&recdata, buffer, size*nmemb);
  return size * nmemb;
}

static void get_id(const char *id, char **ret)
{
  char *v, *w;
  v = strstr(recdata.data, id); if (!v) R;
  while (*v && *v != '"') v++; if (!*v) R;
  v++;
  w = v;
  while (*w && *w != '"') w++; if (!*w) R;
  if (v == w) R;
  *w = 0;
  *ret = strdup(v); if (!*ret) R;
  *w = '"';
}

static void get_chan_id(server_data *d)
{
  char *v, *w;
  v = strstr(recdata.data, "join"); if (!v) R;
  while (*v && *v != '(') v++; if (!*v) R; v++; if (!*v) R; v++;
  w = v;
  while (*w && *w != '\'') w++; if (!*w) R;
  if (v == w) R;
  *w = 0;
  d->chanid = strdup(v); if (!d->chanid) R;
  *w = '\'';
}

static void get_cookie(server_data *d, char *c)
{
  char *v, *w;
  v = c;
  while (*v && *v != '\t') v++; if (!*v) R; v++;
  while (*v && *v != '\t') v++; if (!*v) R; v++;
  while (*v && *v != '\t') v++; if (!*v) R; v++;
  while (*v && *v != '\t') v++; if (!*v) R; v++;
  while (*v && *v != '\t') v++; if (!*v) R; v++;
  w = v;
  while (*w && *w != '\t') w++; if (!*w) R;
  *w = '=';
  reset_buffer(&snddata);
  add_buffer_s(&snddata, v);
  add_buffer_char(&snddata, 0);
  d->cookie = strdup(snddata.data); if (!d->cookie) R;
}

static void _connect(struct chat *c)
{
  server_data *d = (server_data *)c->server_data;
  char url[512];
  CURL *h;
  CURLcode ret;
  struct curl_slist *clist;

  /* get main page */
  h = curl_easy_init(); if (!h) R;
  reset_buffer(&recdata);
  if (strlen(d->channel) + strlen(d->base_url) > 200) R;
  sprintf(url, "%s/%s", d->base_url, d->channel);
  curl_easy_setopt(h, CURLOPT_URL, url);
  curl_easy_setopt(h, CURLOPT_COOKIEFILE, "");
  curl_easy_setopt(h, CURLOPT_WRITEFUNCTION, get_data_cb);
  ret = curl_easy_perform(h); if (ret) R;
  add_buffer_char(&recdata, 0);
  ret = curl_easy_getinfo(h, CURLINFO_COOKIELIST, &clist); if (ret) R;
  curl_easy_cleanup(h);
  if (!clist) R;
  get_cookie(d, clist->data);
  curl_slist_free_all(clist);
  get_id("pfc_clientid", &d->clientid);
  get_id("pfc_nickid", &d->nickid);

  LOG("clientid=%s\n", d->clientid);
  LOG("nickid=%s\n", d->nickid);

  /* connect to chat */
  reset_buffer(&snddata);
  add_buffer_s(&snddata, "pfc_ajax=1&f=handleRequest&cmd=%2Fconnect%20");
  add_buffer_s(&snddata, d->clientid);
  add_buffer_s(&snddata, "%200%20%22");
  add_buffer_s(&snddata, d->login);
  add_buffer_s(&snddata, "%22");
  add_buffer_char(&snddata, 0);
  h = curl_easy_init(); if (!h) R;
  reset_buffer(&recdata);
  if (strlen(d->channel) + strlen(d->base_url) > 200) R;
  sprintf(url, "%s/chatEngine/?room=%s", d->base_url, d->channel);
  curl_easy_setopt(h, CURLOPT_URL, url);
  curl_easy_setopt(h, CURLOPT_COOKIE, d->cookie);
  curl_easy_setopt(h, CURLOPT_WRITEFUNCTION, get_data_cb);
  curl_easy_setopt(h, CURLOPT_POSTFIELDS, snddata.data);
  ret = curl_easy_perform(h); if (ret) R;
  add_buffer_char(&recdata, 0);
  curl_easy_cleanup(h);
  LOG("%s\n", recdata.data);
  get_chan_id(d);
  LOG("chanid=%s\n", d->chanid);
  extract_messages(c, recdata.data);
}

static void _poll(struct chat *c)
{
  server_data *d = (server_data *)c->server_data;
  char url[512];
  CURL *h;
  CURLcode ret;
  int nb;

  /* send update */
  reset_buffer(&snddata);
  add_buffer_s(&snddata, "pfc_ajax=1&f=handleRequest&cmd=%2Fupdate%20");
  add_buffer_s(&snddata, d->nickid);
  add_buffer_s(&snddata, "%20");
  add_buffer_s(&snddata, d->chanid);
  add_buffer_s(&snddata, "%20");
  add_buffer_char(&snddata, 0);
  h = curl_easy_init(); if (!h) R;
  reset_buffer(&recdata);
  if (strlen(d->channel) + strlen(d->base_url) > 200) R;
  sprintf(url, "%s/chatEngine/?room=%s", d->base_url, d->channel);
  curl_easy_setopt(h, CURLOPT_URL, url);
  curl_easy_setopt(h, CURLOPT_COOKIE, d->cookie);
  curl_easy_setopt(h, CURLOPT_WRITEFUNCTION, get_data_cb);
  curl_easy_setopt(h, CURLOPT_POSTFIELDS, snddata.data);
  ret = curl_easy_perform(h); if (ret) R;
  add_buffer_char(&recdata, 0);
  curl_easy_cleanup(h);
  LOG("%s\n", recdata.data);
  acquire_gui();
  nb = extract_messages(c, recdata.data);
  if (nb)
    d->no_activity = 0;
  else if (d->no_activity < d->switch_delay)
    d->no_activity += d->fast_delay;
  if (d->no_activity >= d->switch_delay) {
    LOG("slow delay, no activity is big\n");
    gui_update_timer_start(c->gui, d->slow_delay);
  } else {
    LOG("fast delay, no activity is small\n");
    gui_update_timer_start(c->gui, d->fast_delay);
  }
  release_gui();
}

static void _disconnect(struct chat *c)
{
  /* TODO: do a /quit or /leave or whatever */
}

static void _send(struct chat *c, char *m)
{
  server_data *d = (server_data *)c->server_data;
  char url[512];
  CURL *h;
  CURLcode ret;
  int i;

  m = encode_message(c, m);
  LOG("SSSSSSSSSSSSSSSSSSSSSS: %s\n", m);

  /* send 'send' */
  reset_buffer(&snddata);
  add_buffer_s(&snddata, "pfc_ajax=1&f=handleRequest&cmd=%2Fsend%20");
  add_buffer_s(&snddata, d->nickid);
  add_buffer_s(&snddata, "%20");
  add_buffer_s(&snddata, d->chanid);
  add_buffer_s(&snddata, "%20");
  add_buffer_char(&snddata, '_');
  add_buffer_char(&snddata, '_');
  for (i = 0; i < 92; i++) add_buffer_char(&snddata, m[i]);
  add_buffer_char(&snddata, 0);
  h = curl_easy_init(); if (!h) R;
  reset_buffer(&recdata);
  if (strlen(d->channel) + strlen(d->base_url) > 200) R;
  sprintf(url, "%s/chatEngine/?room=%s", d->base_url, d->channel);
  curl_easy_setopt(h, CURLOPT_URL, url);
  curl_easy_setopt(h, CURLOPT_COOKIE, d->cookie);
  curl_easy_setopt(h, CURLOPT_WRITEFUNCTION, get_data_cb);
  curl_easy_setopt(h, CURLOPT_POSTFIELDS, snddata.data);
  ret = curl_easy_perform(h); if (ret) R;
  add_buffer_char(&recdata, 0);
  curl_easy_cleanup(h);
  LOG("%s\n", recdata.data);
  acquire_gui();
  extract_messages(c, recdata.data);
  release_gui();
}

static void _send_over(struct chat *c)
{
  server_data *d = c->server_data;
  acquire_gui();
  d->no_activity = 0;
  gui_update_timer_start(c->gui, d->fast_delay);
  release_gui();
}

static int _new_message(chat *c, const char *s)
{
  /* nothing to do in mathim */
  return 1;
}

void mathim_server_init(chat *c)
{
  server_data *d;
  CURLcode ret;
  ret = curl_global_init(CURL_GLOBAL_ALL); if (ret) R;
  init_buffer(&recdata);
  init_buffer(&snddata);
  c->connect = _connect;
  c->poll = _poll;
  c->disconnect = _disconnect;
  c->send = _send;
  c->send_over = _send_over;
  c->new_message = _new_message;
  d = malloc(sizeof(server_data)); if (!d) R;
  c->server_data = d;
  d->base_url = "http://www.mathim.com";
//  d->base_url = "http://www.psychicschoolhouse.com/phpfreechat/demo/demo21_with_hardcoded_urls.php";
  d->channel = c->channel;
  d->login = get_option(&c->options, "connection.mathim.login");
  sscanf(get_option(&c->options, "connection.mathim.fast"), "%d",
         &d->fast_delay);
  sscanf(get_option(&c->options, "connection.mathim.slow"), "%d",
         &d->slow_delay);
  sscanf(get_option(&c->options, "connection.mathim.switch"), "%d",
         &d->switch_delay);
  d->no_activity = 0;
  if (d->fast_delay < 1 || d->fast_delay > 100 ||
      d->slow_delay < 1 || d->slow_delay > 100 ||
      d->switch_delay < 1 || d->switch_delay > 100) {
    printf("bad mathim delays in configuration file\n");
    exit(1);
  }
}
