#include "options.h"

#include "defs.h"
#include "mathim/buffer.h"

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>

static do_buf obuf;

static char *get_line(FILE *f, const char *fn)
{
  int c;
  int pos = 0;

  reset_buffer(&obuf);

com:
  c = getc(f); if (c != EOF) ungetc(c, f);
  if (c == '#') {
    do c = getc(f); while (c != EOF && c != '\n');
    goto com;
  }

  while (1) {
    c = getc(f);
    if (c == EOF) break;
    if (c == '\n') break;
    if (isspace(c) && pos == 0) { c++; continue; }
    if (c == '=' && pos == 0) {
      add_buffer_char(&obuf, 0);
      pos = obuf.size;
      if (pos == 1) goto err;
      do c = getc(f); while (c != EOF && c != '\n' && isspace(c));
      if (c != EOF) ungetc(c, f);
      continue;
    }
    add_buffer_char(&obuf, c);
  }

  if (obuf.size == 0) {
    if (feof(f)) return 0;
    goto com;
  }

  add_buffer_char(&obuf, 0);

  if (pos == 0 || obuf.data[pos] == 0) goto err;

  return obuf.data + pos;

err:
    printf("%s: bad configuration file\n", fn);
    exit(1);
}

static void add_option(options *o, const char *name, const char *val)
{
  char *n = strdup(name);
  char *v = strdup(val);
  if (!n || !v) R;
  if (o->size == o->maxsize) {
    o->data = realloc(o->data, sizeof(option) * (o->maxsize += 32));
    if (!o->data) R;
  }
  o->data[o->size].name = n;
  o->data[o->size].val = v;
  o->size++;
}

void read_options(options *o, const char *conffile, char **v, int n)
{
  char *c;
  char *home;
  FILE *f;
  int i;

  o->data = 0;
  o->size = o->maxsize = 0;

  init_buffer(&obuf);

  if (!conffile) {
    home = getenv("HOME");
    if (!home) {
      printf("environment variable HOME not set\n");
      exit(1);
    }
    c = malloc(strlen(home) + strlen("/.chat.conf") + 1); if (!c) R;
    sprintf(c, "%s/.chat.conf", home);
    f = fopen(c, "rb");
    if (!f) { printf("cannot open configuration file %s\n", c); exit(1); }
  } else {
    f = fopen(conffile, "rb");
    if (!f) {printf("cannot open configuration file %s\n",conffile);exit(1);}
  }

  while (1) {
    char *val = get_line(f, conffile?conffile:c);
    if (!val) break;
    add_option(o, obuf.data, val);
  }

  fclose(f);

  if (!conffile) free(c);

  free_buffer(&obuf);

  /* overwrite option if passed from command line */
  for (i = 1; i < n; i++) {
    int j;
    if (!v[i]) continue;
    for (j = 0; j < o->size; j++) if (!strcmp(o->data[j].name, v[i])) break;
    if (j == o->size) continue;
    if (i == n-1) continue;
    free(o->data[j].val);
    o->data[j].val = strdup(v[i+1]); if (!o->data[j].val) R;
    v[i] = 0;
    v[i+1] = 0;
    i++;
  }
}

char *get_option(options *o, const char *n)
{
  int i;

  for (i = 0; i < o->size; i++) {
    if (!strcmp(o->data[i].name, n)) return o->data[i].val;
  }

  printf("%s: unknown option, check the configuration file\n", n);
  exit(1);
}
